package middleware

import (
	jwt "github.com/appleboy/gin-jwt/v2"
	"github.com/gin-gonic/gin"
	"github.com/go-pg/pg/v10"
	"service-status-backend/apperr"
	"service-status-backend/config"
	"service-status-backend/model"
	"service-status-backend/request"
	"service-status-backend/utils"
	"time"
)

var IdentityKey = "schacc"

// AuthMiddleware Returns a preconfigured JWT Auth Middleware
func AuthMiddleware(DB *pg.DB) (*jwt.GinJWTMiddleware, error) {
	conf := config.GetJWTConfig()

	authMiddleware, err := jwt.New(&jwt.GinJWTMiddleware{
		Realm:       conf.AuthRealm,
		Key:         []byte(conf.AuthKey),
		Timeout:     time.Hour * time.Duration(conf.AuthTimeoutHours),
		MaxRefresh:  time.Hour * time.Duration(conf.AuthMaxRefreshHours),
		IdentityKey: IdentityKey,

		PayloadFunc: func(data interface{}) jwt.MapClaims {
			if v, ok := data.(*model.User); ok {
				return jwt.MapClaims{
					IdentityKey: v.Schacc,
					"email":     v.Email,
					"fullName":  v.FullName,
					"id":        v.Id,
				}
			}
			return jwt.MapClaims{}
		},

		IdentityHandler: func(c *gin.Context) interface{} {
			claims := jwt.ExtractClaims(c)
			return &model.User{
				Schacc: claims[IdentityKey].(string),
			}
		},

		Authenticator: func(c *gin.Context) (interface{}, error) {
			var loginValues request.Login
			if err := c.ShouldBind(&loginValues); err != nil {
				return nil, jwt.ErrMissingLoginValues
			}

			// Check given data
			UserName := loginValues.Username
			password := loginValues.Password

			foundUser := new(model.User)
			err := DB.Model(foundUser).Where("schacc = ?", UserName).Select()

			if err == nil && utils.CheckPasswordHash(password, foundUser.Password) {

				if !foundUser.IsApproved {
					return nil, apperr.New(403, "User is not approved!")
				}

				return foundUser, nil
			}

			return nil, jwt.ErrFailedAuthentication
		},

		Authorizator: func(data interface{}, c *gin.Context) bool {
			_, ok := data.(*model.User)
			return ok
		},

		Unauthorized: func(c *gin.Context, code int, message string) {
			c.JSON(code, gin.H{
				"code":    code,
				"message": message,
			})
		},

		// TokenLookup is a string in the form of "<source>:<name>" that is used
		// to extract token from the request.
		// Optional. Default value "header:Authorization".
		// Possible values:
		// - "header:<name>"
		// - "query:<name>"
		// - "cookie:<name>"
		// - "param:<name>"
		TokenLookup: "header: Authorization, query: token, cookie: jwt",
		// TokenLookup: "query:token",
		// TokenLookup: "cookie:token",

		// TokenHeadName is a string in the header. Default value is "Bearer"
		TokenHeadName: "Bearer",

		// TimeFunc provides the current time. You can override it to use another time value. This is useful for testing or if your server uses a different time zone than your tokens.
		TimeFunc: time.Now,
	})

	return authMiddleware, err
}
