package route

import (
	"github.com/gin-gonic/gin"
	"github.com/go-pg/pg/v10"
	"github.com/swaggo/files"       // swagger embed files
	"github.com/swaggo/gin-swagger" // gin-swagger middleware
	"go.uber.org/zap"
	_ "service-status-backend/docs"
	"service-status-backend/middleware"
	"service-status-backend/repository"
	"service-status-backend/svc"
)

// NewServices creates a new router services
func NewServices(DB *pg.DB, Log *zap.Logger, R *gin.Engine) *Services {
	return &Services{DB, Log, R}
}

// Services lets us bind specific services when setting up routes
type Services struct {
	DB  *pg.DB
	Log *zap.Logger
	R   *gin.Engine
}

func (s *Services) SetupV1Routes() {
	// Auth
	jwt, err := middleware.AuthMiddleware(s.DB)
	if err != nil {
		s.Log.Fatal("Couldn't configure JWT Middleware", zap.Error(err))
	}

	// database logic
	userRepo := repository.NewUserRepo(s.DB, s.Log)
	serviceRepo := repository.NewServiceRepo(s.DB, s.Log)
	historyRepo := repository.NewHistoryRepo(s.DB, s.Log)

	// svc logic
	userSVC := svc.NewUserSVC(userRepo, s.Log)
	serviceSVC := svc.NewServiceSVC(serviceRepo, s.Log)
	historySVC := svc.NewHistorySVC(historyRepo, s.Log)

	// prefixed with /api/v1
	v1Router := s.R.Group("/api/v1")
	v1Router.GET("/swagger/*any", ginSwagger.WrapHandler(swaggerFiles.Handler))
	SetAuthRouter(v1Router, jwt)
	SetUserRouter(userSVC, v1Router, jwt)
	SetServiceRoutes(serviceSVC, v1Router, jwt)
	SetHistoryRoutes(historySVC, v1Router, jwt)
}
