package server

import (
	"github.com/gin-contrib/cors"
	"github.com/gin-gonic/gin"
	"github.com/go-pg/pg/extra/pgdebug"
	"go.uber.org/zap"
	"service-status-backend/config"
	"service-status-backend/model"
	"service-status-backend/route"
	"strconv"
	"time"
)

// Server holds all the routes and their services
type Server struct {
	RouteServices []route.ServicesI
}

// Run runs our API server
func (server *Server) Run() error {
	c := config.GetAppConfig()

	r := gin.Default()
	r.Use(cors.New(cors.Config{
		AllowOrigins:     []string{"*"},
		AllowMethods:     []string{"GET", "POST", "PUT", "DELETE", "OPTIONS"},
		AllowHeaders:     []string{"Origin", "X-Requested-With", "Content-Length", "Content-Type", "Authorization"},
		AllowCredentials: false,
		MaxAge:           12 * time.Hour,
	}))

	// middleware
	db := config.GetConnection()
	db.AddQueryHook(pgdebug.DebugHook{
		// Print all queries
		Verbose: c.IsPGDebug,
	})
	model.CreateSchema(db)

	log, _ := zap.NewDevelopment()
	defer log.Sync()

	// setup default routes
	rsDefault := &route.Services{
		DB:  db,
		Log: log,
		R:   r}
	rsDefault.SetupV1Routes()

	// setup all custom/user-defined route services
	for _, rs := range server.RouteServices {
		rs.SetupRoutes()
	}

	// run with port from config
	return r.Run(":" + strconv.Itoa(c.Port))
}
