package svc

import (
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"net/http"
	"service-status-backend/apperr"
	"service-status-backend/model"
	"service-status-backend/request"
	"service-status-backend/response"
)

// HistorySVC represents the service svc
type HistorySVC struct {
	historyRepo model.HistoryRepo
	log         *zap.Logger
}

// NewUserSVC create a new service application svc
func NewHistorySVC(historyRepo model.HistoryRepo, log *zap.Logger) *HistorySVC {
	return &HistorySVC{
		historyRepo: historyRepo,
		log:         log,
	}
}

// Create creates a new History
// @Summary Create a History
// @Description Creates a history for the given Service. The serviceId has to be an existing service.
// @Security ApiKeyAuth
// @Accept  json
// @Produce  json
// @Tags history
// @Param history body request.HistoryCreateStruct true "Add History"
// @Success 201 {object} response.Created
// @Router /histories [post]
func (s *HistorySVC) Create(c *gin.Context) {
	r, err := request.HistoryCreate(c)
	if err != nil {
		return
	}

	history := &model.History{
		CreatedAt:   r.CreatedAt,
		Description: r.Description,
		ServiceId:   r.ServiceId,
	}
	if err := s.historyRepo.Create(history); err != nil {
		apperr.Response(c, err)
		return
	}
	c.JSON(http.StatusCreated, response.Created{Id: history.Id})
}

// Update updates a history
// @Summary Update a History
// @Description Updates a history. The new serviceId has to be an existing service.
// @Security ApiKeyAuth
// @Accept  json
// @Produce  json
// @Tags history
// @Param history body request.HistoryUpdateStruct true "Update History"
// @Param historyId path int true "History ID"
// @Success 200
// @Router /histories/{historyId} [put]
func (s *HistorySVC) Update(c *gin.Context) {
	id, err := request.ID(c)
	if err != nil {
		return
	}
	r, err := request.HistoryUpdate(c)
	if err != nil {
		return
	}

	history := &model.History{
		Id:          id,
		CreatedAt:   r.CreatedAt,
		Description: r.Description,
		ServiceId:   r.ServiceId,
	}
	if _, err := s.historyRepo.Update(history); err != nil {
		apperr.Response(c, err)
		return
	}
	c.Status(http.StatusOK)
}

// Delete deletes a History
// @Summary Delete a History
// @Description Deletes a given history.
// @Security ApiKeyAuth
// @Accept  json
// @Produce  json
// @Tags history
// @Param historyId path int true "History ID"
// @Success 200
// @Router /histories/{historyId} [delete]
func (s *HistorySVC) Delete(c *gin.Context) {
	id, err := request.ID(c)
	if err != nil {
		return
	}

	if err := s.historyRepo.Delete(id); err != nil {
		apperr.Response(c, err)
		return
	}
	c.Status(http.StatusOK)
}
