package svc

import (
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"net/http"
	"service-status-backend/apperr"
	"service-status-backend/model"
	"service-status-backend/request"
	"service-status-backend/response"
)

// HistorySVC represents the service svc
type ServiceSVC struct {
	serviceRepo model.ServiceRepo
	log         *zap.Logger
}

// NewUserSVC create a new service application svc
func NewServiceSVC(serviceRepo model.ServiceRepo, log *zap.Logger) *ServiceSVC {
	return &ServiceSVC{
		serviceRepo: serviceRepo,
		log:         log,
	}
}

// View returns a Service
// @Summary View returns a Service with history
// @Description Return a service with the history
// @Accept  json
// @Produce  json
// @Tags service
// @Param serviceId path int true "Service ID"
// @Success 200 {object} model.Service
// @Router /services/{serviceId} [get]
func (s *ServiceSVC) View(c *gin.Context) {
	id, err := request.ID(c)
	if err != nil {
		return
	}
	result, err := s.serviceRepo.View(id)
	if err != nil {
		apperr.Response(c, err)
		return
	}
	c.JSON(http.StatusOK, result)
}

// List returns list of Services
// @Summary List Services
// @Description get services. Histories isn't sent with it.
// @Accept  json
// @Produce  json
// @Tags service
// @Success 200 {array} model.Service
// @Router /services [get]
func (s *ServiceSVC) List(c *gin.Context) {
	result, err := s.serviceRepo.List()
	if err != nil {
		apperr.Response(c, err)
		return
	}
	c.JSON(http.StatusOK, result)
}

// Create creates a new Service
// @Summary Create a Service
// @Description Create a Service
// @Security ApiKeyAuth
// @Accept  json
// @Produce  json
// @Tags service
// @Param service body request.ServiceCreateStruct true "Add Service"
// @Success 201 {object} response.Created
// @Router /services [post]
func (s *ServiceSVC) Create(c *gin.Context) {
	r, err := request.ServiceCreate(c)
	if err != nil {
		return
	}
	service := &model.Service{
		Name:        r.Name,
		Description: r.Description,
		IsOkay:      r.IsOkay,
	}
	if err := s.serviceRepo.Create(service); err != nil {
		apperr.Response(c, err)
		return
	}
	c.JSON(http.StatusCreated, response.Created{Id: service.Id})
}

// Delete deletes a Service
// @Summary Delete a Service
// @Description Deletes a service with all of it's History.
// @Security ApiKeyAuth
// @Accept  json
// @Produce  json
// @Tags service
// @Param serviceId path int true "Service ID"
// @Success 200
// @Router /services/{serviceId} [delete]
func (s *ServiceSVC) Delete(c *gin.Context) {
	id, err := request.ID(c)
	if err != nil {
		return
	}

	if err := s.serviceRepo.Delete(id); err != nil {
		apperr.Response(c, err)
		return
	}
	c.Status(http.StatusOK)
}

// Update updates a Service
// @Summary Update a Service
// @Description Update a Service.
// @Security ApiKeyAuth
// @Accept  json
// @Produce  json
// @Tags service
// @Param service body request.ServiceUpdateStruct true "Update Service"
// @Param serviceId path int true "Service ID"
// @Success 200
// @Router /services/{serviceId} [put]
func (s *ServiceSVC) Update(c *gin.Context) {
	id, err := request.ID(c)
	if err != nil {
		return
	}
	r, err := request.ServiceUpdate(c)
	if err != nil {
		return
	}

	service := &model.Service{
		Id:          id,
		Description: r.Description,
		Name:        r.Name,
		IsOkay:      r.IsOkay,
	}
	_, err = s.serviceRepo.Update(service)
	if err != nil {
		apperr.Response(c, err)
		return
	}
	c.Status(http.StatusOK)
}
