package svc

import (
	"github.com/gin-gonic/gin"
	"go.uber.org/zap"
	"net/http"
	"service-status-backend/apperr"
	"service-status-backend/model"
	"service-status-backend/request"
	"service-status-backend/response"
	"service-status-backend/utils"
)

// UserSVC represents the user svc
type UserSVC struct {
	userRepo model.UserRepo
	log      *zap.Logger
}

// NewUserSVC create a new user application svc
func NewUserSVC(userRepo model.UserRepo, log *zap.Logger) *UserSVC {
	return &UserSVC{
		userRepo: userRepo,
		log:      log,
	}
}

// View returns a User
// @Summary View returns a User
// @Description Return a User
// @Security ApiKeyAuth
// @Accept  json
// @Produce  json
// @Tags user
// @Param userId path int true "User ID"
// @Success 200 {object} model.User
// @Router /users/{userId} [get]
func (s *UserSVC) View(c *gin.Context) {
	id, err := request.ID(c)
	if err != nil {
		return
	}
	result, err := s.userRepo.View(id)
	if err != nil {
		apperr.Response(c, err)
		return
	}
	c.JSON(http.StatusOK, result)
}

// List returns list of Users
// @Summary List Users
// @Description get Users
// @Security ApiKeyAuth
// @Accept  json
// @Produce  json
// @Tags user
// @Success 200 {array} model.User
// @Router /users [get]
func (s *UserSVC) List(c *gin.Context) {
	result, err := s.userRepo.List()
	if err != nil {
		apperr.Response(c, err)
		return
	}
	c.JSON(http.StatusOK, result)
}

// Create creates a new User (Registration)
// @Summary Create User (Registration)
// @Description create User
// @Accept  json
// @Produce  json
// @Tags user
// @Param user body request.UserRegisterStruct true "Add User"
// @Success 201 {object} response.Created
// @Router /users [post]
func (s *UserSVC) Create(c *gin.Context) {
	r, err := request.UserCreate(c)
	if err != nil {
		return
	}
	hashedPassword, _ := utils.HashPassword(r.Password)
	user := &model.User{
		Email:      r.Email,
		FullName:   r.FullName,
		Password:   hashedPassword,
		Schacc:     r.Schacc,
		IsApproved: false,
	}
	if err := s.userRepo.Create(user); err != nil {
		apperr.Response(c, err)
		return
	}
	c.JSON(http.StatusCreated, response.Created{Id: user.Id})
}

// Approve approves the Given User
// @Summary Approve User (Can login)
// @Description Approve User. Can login only when approved.
// @Security ApiKeyAuth
// @Accept  json
// @Produce  json
// @Tags user
// @Param userId path int true "User ID"
// @Success 200 {object} string ok
// @Router /approve_user/{userId} [post]
func (s *UserSVC) Approve(c *gin.Context) {
	id, err := request.ID(c)
	if err != nil {
		return
	}
	user := &model.User{
		Id:         id,
		IsApproved: true,
	}
	_, err = s.userRepo.Update(user)
	if err != nil {
		apperr.Response(c, err)
		return
	}
	c.Status(http.StatusOK)
}
